% ICIP 2013:
% << Feature Normalization for Part-Based Image Classification >>
% by Lingxi Xie, Qi Tian and Bo Zhang.
% ---------------------------------------------------------------------------
% Program Name:         Classifier_HRC
% Program Description:  Hierarchical feature normalization program with adjustable parameters: power and coefficient.
% ---------------------------------------------------------------------------
% File Name:            Main.m
% File Description:     Defining parameters, paths and outlining the program.
% ---------------------------------------------------------------------------
% Code by Lingxi Xie, Sep, 2013.

%% 1. Definition of Parameters and Names.

% Path name of the features.
FEATURE_NAME = 'CUB-200-2011';

% The structure of super-parts (see the Hierarchical Part Matching paper for reference).
% In the CUB-200-2011 dataset, 15 parts are annotated on each bird:
%  1. back        2. beak        3. belly       4. breast      5. crown
%  6. forehead    7. left-eye    8. left-leg    9. left-wing  10. nape
% 11. right-eye  12. right-leg  13. right-wing 14. tail       15. throat
% Moreover, we can specify or learn some super-parts to describe the high-level structures, such as:
%   #1 eyes  = left-eye (7) + right-eye (11);
%   #2 head  = beak (2) + crown (5) + forehead (6) + left-eye (7) + right-eye (11);
%   #3 neck  = nape (10) + throat (15);
%   #4 body  = back (1) + belly (3) + breast (4);
%   #5 wings = left-wing (9) + right-wing (13);
%   #6 legs  = left-leg (8) + right-wing(12).
% We define the super-part structure above as a matrix.
% Note that these settings are related to the RAW features specified.
% If you want to use other settings, please use the correct RAW features.
% Indeed, we only care about the number of non-zero elements in each row, see the codes for reference.
SUPERPART = [
    7, 11, 0, 0, 0;
    2, 5, 6, 7, 11;
    10, 15, 0, 0, 0;
    1, 3, 4, 0, 0;
    9, 13, 0, 0, 0;
    8, 12, 0, 0, 0];
% The structure of part-based features.
% In general, part-based features are generated from a number of (P) pooling bins:
% 1st: [w11, w12, w13, ..., w1K]
% 2nd: [w21, w22, w23, ..., w2K]
% 3rd: [w31, w32, w33, ..., w3K]
% ......
% Pth: [wP1, wP2, wP3, ..., wPK]
% According to the implementation details, the features might be concatenated in the following ways:
% ROW type (ROW_TYPE = true):
% [w11, w12, w13, ..., w1K, w21, w22, w23, ..., w2K, ..., w31, w32, w33, ..., w3K, ..., wP1, wP2, wP3, ..., wPK]
% or COLUMN type (ROW_TYPE = false):
% [w11, w21, w31, ..., wP1, w12, w22, w32, ..., wP2, ..., w13, w23, w33, ..., wP3, ..., w1K, w2K, w3K, ..., wPK]
% Please choose the type based on your implementation below.
ROW_TYPE = false;
% The size of each pooling bin: KEY information for separate normalization.
BIN_SIZE = 2048;

% Path name of the features to normalize, where 'H' means 'hierarchical model'.
RAWDATA_NAME = 'RAW';
% Path of the features to normalize.
FEATURE_NAME_1 = fullfile(FEATURE_NAME, RAWDATA_NAME);
% Parameters in normalization.
% POWER: the normalization power (-1 is for L_inf-norm).
% COEF: the normalization coefficient.
POWER = 2;
COEF = 1;
% Path name of the normalized features.
NORMALIZATION_NAME = ['L(', num2str(POWER), ')_' num2str(COEF), 'C,HRC'];
% Path of the normalized features.
FEATURE_NAME_2 = fullfile(FEATURE_NAME, NORMALIZATION_NAME);

% Classification settings.
% We use the LibLINEAR, a linear SVM for scalable image classification.
% Rounds of random dataset split. Average classification accuracy is reported.
TIMES = 10;
% Images per category for training. All the remaining images are used for testing.
TRAINING_SAMPLES_PER_CATEGORY = 5;
% The slacking variable C of SVM.
% You can also use multiple values for C for classification and choose the best one.
% Usage: C = [5, 8, 10, 12, 20];
C = 10;
% Path name of classification settings (same as the feature path).
RESULT_NAME = FEATURE_NAME_2;
% File name of classification settings.
RESULT_FILE_NAME = ['T', num2str(TRAINING_SAMPLES_PER_CATEGORY), '_C'];
RESULT_FILE_EXPANSION = '.txt';

%% 2. Definition of Paths.

% Data path: absolute path is used.
DATA_PATH = 'F:\data';
% Paths corresponding to the names defined above.
FEATURE_PATH_1 = fullfile(DATA_PATH, 'nmfeatures', FEATURE_NAME_1);
FEATURE_PATH_2 = fullfile(DATA_PATH, 'nmfeatures', FEATURE_NAME_2);
RESULT_PATH = fullfile(DATA_PATH, 'nmresults', RESULT_NAME);

%% 3. Outlining the program.

% Featrue normalization.
FeatureNormalization(SUPERPART, FEATURE_PATH_1, FEATURE_PATH_2, ROW_TYPE, BIN_SIZE, POWER, COEF);
% Classifying images using the normalized features.
Classifier(FEATURE_PATH_2, RESULT_PATH, RESULT_FILE_NAME, ...
        RESULT_FILE_EXPANSION, TIMES, TRAINING_SAMPLES_PER_CATEGORY, C);
